package com.example.dashboard;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.stream.annotation.StreamListener;
import org.springframework.messaging.handler.annotation.Payload;
import org.springframework.stereotype.Service;

import java.util.Optional;

@Service
public class EventHandler {
    @Autowired ProductRepository productRepository;
    @Autowired OrderRepository orderRepository;

    @StreamListener(StreamProcessor.INPUT)
    public void onProductCreated_recordDashboard(@Payload ProductCreated productCreated){

        if(!productCreated.isMe()) return;
        System.out.println("##### ProductCreated in Dashboard : " + productCreated.toJson());

        Product product  = new Product();

        product.setProductId(productCreated.getProductId());
        product.setProductName(productCreated.getProductName());
        product.setProductStock(productCreated.getProductStock());
        productRepository.save(product);

    }

    @StreamListener(StreamProcessor.INPUT)
    public void onOrderPlaced_recordDashboard(@Payload OrderPlaced orderPlaced){

        if(!orderPlaced.isMe()) return;
        System.out.println("##### OrderPlaced in Dashboard: " + orderPlaced.toJson());

        Order order  = new Order();

        order.setProductId(orderPlaced.getProductId());
        order.setOrderId(orderPlaced.getId());
        order.setOrderQty(orderPlaced.getQty());
        order.setOrderPrice(orderPlaced.getPrice());
        order.setOrderStatus("** Ordered **");
        order.setTimestamp(orderPlaced.getTimestamp());
        orderRepository.save(order);

        Product product  = productRepository.findByProductId(orderPlaced.getProductId());
        if (product != null) {
            product.setProductStock(product.getProductStock() - orderPlaced.getQty());
            productRepository.save(product);
        }

    }

    @StreamListener(StreamProcessor.INPUT)
    public void onOrderCancelled_recordDashboard(@Payload OrderCancelled orderCancelled){

        if(!orderCancelled.isMe()) return;
        System.out.println("##### OrderPlaced in Dashboard: " + orderCancelled.toJson());

        Order order  = new Order();

        order.setProductId(orderCancelled.getProductId());
        order.setOrderId(orderCancelled.getId());
        order.setOrderQty(orderCancelled.getQty());
        order.setOrderPrice(orderCancelled.getPrice());
        order.setOrderStatus("** Cancelled **");
        order.setTimestamp(orderCancelled.getTimestamp());
        orderRepository.save(order);

        Product product  = productRepository.findByProductId(orderCancelled.getProductId());
        if (product != null) {
            product.setProductStock(product.getProductStock() + orderCancelled.getQty());
            productRepository.save(product);
        }
    }

}
