package com.example.order;

import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.net.http.HttpResponse;
import java.security.InvalidParameterException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Optional;

@RestController
public class OrderController {

    @Autowired OrderRepository orderRepository;

    @RequestMapping(value = "/orders",
            method = RequestMethod.POST,
            produces = "application/json;charset=UTF-8")
    public void placeOrder(@RequestBody HashMap<String, String> map) throws Exception {
        Long productId = Util.getParam(Long.class, map,"productId", true);
        Long userId    = Util.getParam(Long.class, map,"userId", false);

        int  price = Util.getParam(Integer.class, map,"price", false);
        int  qty   = Util.getParam(Integer.class, map,"qty", false);

        OrderApplication.applicationContext.getBean(ProductService.class)
                .decreaseStock(productId, qty);

        Order order = new Order();

        try {
            order.setProductId(productId);
            order.setQty(qty);
            if (userId != 0) order.setUserId(userId);
            if (price != 0) order.setPrice(price);

            if (qty <= 0) throw new RuntimeException("qty value (" + qty + ") is wrong !!  ");

            // for Test
            if (price <0) throw new RuntimeException("DUMMY EXCEPTION for Compensation Test");
            //order = null;

            orderRepository.save(order);

        } catch (Exception e) {
            System.out.println("\n $$$ Exception occurred in creating order ==> " + e.getMessage());

            // 에러 발생시, 상품 재고를 다시 원복시켜 주어야 함
            OrderCancelled orderCancelled = new OrderCancelled();
            BeanUtils.copyProperties(order, orderCancelled);
            orderCancelled.publish();
            System.out.println("\n $$$ Published <OrderCancelled> Event ");

        }
    }

    @GetMapping({"/orders", "/orders/{orderId}"})
    public ResponseEntity<Object> getOrder(@PathVariable(required = false) Long orderId) throws Exception {

        List<Order> orderList = new ArrayList<>();

        if ( orderId == null) {
            orderList = orderRepository.findAll();
        } else {
            Optional<Order> optOrder = orderRepository.findById(orderId);
            optOrder = orderRepository.findById(orderId);
            if (!optOrder.isEmpty()) orderList.add(optOrder.get());
        }

        if (orderList.isEmpty()) {
            return ResponseEntity.status(HttpStatus.NOT_FOUND).body("");
        }
        return ResponseEntity.ok().body(orderList);
    }


    @RequestMapping(value = "/orders/{orderId}",
            method = RequestMethod.DELETE,
            produces = "application/json;charset=UTF-8")
    public void cancelOrder(@PathVariable long orderId) throws Exception {

        Optional<Order> optOrder = orderRepository.findById(orderId);
        if (optOrder.isEmpty()) return;
        Order order = optOrder.get();
        orderRepository.delete(order);
    }

}
